/*
	msettingsdialog.cpp - settings dialog
	Copyright (C) 2003  Konrad Twardowski <kdtonline@poczta.onet.pl>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "configuration.h"
#include "extras.h"
#include "links.h"
#include "mactioneditdialog.h"
#include "miscutils.h"
#include "mmessagedialog.h"
#include "msettingsdialog.h"
#include "msystemtray.h"
#include "mtip.h"

#include <qlayout.h>
#include <qtimer.h>
#include <qvgroupbox.h>
#include <qwhatsthis.h>

#include <kconfig.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <knotifydialog.h>
#include <krun.h>
#include <kstandarddirs.h>

/* constructor */

MSettingsDialog::MSettingsDialog(QWidget *parent, const char *name)
	: KDialogBase(
		IconList,
		i18n("Settings"),
		Apply | Cancel | Default | Help | Ok,
		Ok, // default button
		parent,
		name,
		TRUE, // modal
		TRUE // separator
	)
{
	initGeneralPage();
	initNotifyPage();
	initAutoPage();
	initConfirmPage();
	initActionsPage();
	initSchedulerPage();
	initStatsPage();

	connect(
		this, SIGNAL(aboutToShowPage(QWidget*)),
		this, SLOT(aboutToShowPageSlot(QWidget*)));
	connect(
		this, SIGNAL(applyClicked()),
		this, SLOT(okClickedSlot()));
	connect(
		this, SIGNAL(okClicked()),
		this, SLOT(okClickedSlot()));

	// general
	c_timeRemember->setChecked(kshutdownrc->timeRemember);
	// automation
	c_autoLock->setChecked(QFile::exists(KGlobalSettings::autostartPath() +
		"/" + AUTO_LOCK_FILE_NAME));
	c_runCommandBeforeShutDown->setChecked(
		kshutdownrc->runCommandBeforeShutDown
	);
	in_commandBeforeShutDown->setURL(kshutdownrc->commandBeforeShutDown);
	in_commandBeforeShutDown->setEnabled(
		c_runCommandBeforeShutDown->isChecked()
	);
	in_commandPause->setValue(kshutdownrc->commandPause);
	in_commandPause->setEnabled(c_runCommandBeforeShutDown->isChecked());
	// confirmations
	c_confirmAction->setChecked(kshutdownrc->confirmAction);
	c_popupMessages->setChecked(kshutdownrc->popupMessages);
	in_popupTimeout->setValue(kshutdownrc->popupTimeout);
	setWarningMessage(
		kshutdownrc->warningMessageEnabled,
		kshutdownrc->warningMessageDelay
	);
	// scheduler
	c_schedulerEnabled->setChecked(kshutdownrc->schedEnabled);
	// statistics
	c_statsLongFormat->setChecked(kshutdownrc->statsLongFormat);
	b_statsBackground->setColor(kshutdownrc->statsBackground);
	b_statsHeader->setColor(kshutdownrc->statsHeader);
	b_statsText->setColor(kshutdownrc->statsText);
	b_statsWarning->setColor(kshutdownrc->statsWarning);
}

/* destructor */

MSettingsDialog::~MSettingsDialog()
{
}

/* public */

void MSettingsDialog::showPageByName(const QString &name)
{
	if (name.isNull())
		return;

	if (name == "General")
		showPage(pageIndex(_generalPage));
	else if (name == "Confirmations")
		showPage(pageIndex(_confirmPage));
	else if (name == "Actions")
		showPage(pageIndex(_actionsPage));
	else if (name == "Scheduler")
		showPage(pageIndex(_schedulerPage));
	else if (name == "Statistics")
		showPage(pageIndex(_statsPage));
}

/* private */

QFrame *MSettingsDialog::addPage(const QString &itemName, const QString &iconName)
{
	return KDialogBase::addPage(itemName, QString::null, MainBarIcon(iconName, 32));
}

bool MSettingsDialog::disabledByAdmin(const QString &key, QFrame *page,
	QBoxLayout *pageLayout) const
{
	if (mainWindow->isRestricted("kshutdown_settings_" + key))
	{
		// add tip
		if (page)
		{
			MTip *t_admin =
				new MTip(MTip::TipType_Info, page, "MTip::t_admin");
			t_admin->setTipText(
				i18n(
					"This page has been disabled " \
					"by the Administator."
				)
			);
			pageLayout->addWidget(t_admin);
		}

		return TRUE; // disabled
	}

	return FALSE; // enabled
}

void MSettingsDialog::initActionsPage()
{
	_actionsPage = addPage(i18n("Actions & Extras Menu"), "exit");
	QVBoxLayout *l = new QVBoxLayout(_actionsPage, 2);

	if (disabledByAdmin("actions", _actionsPage, l))
		return;

	QVGroupBox *gb_general = new QVGroupBox(i18n("Ac&tions"), _actionsPage,
		"QVGroupBox::gb_general");

	_actions = new QListView(gb_general, "QListView::_actions");
	_actions->setAllColumnsShowFocus(TRUE);
	_actions->setItemMargin(5);
	_actions->setMaximumHeight(200);
	_actions->setSorting(-1); // no sort
	_actions->addColumn(i18n("Action"));
	_actions->addColumn(i18n("Method / Command"));

	_logoutItem = new QListViewItem(_actions,
		mainWindow->getActionName(MMainWindow::What_Logout));
	_logoutItem->setPixmap(
		0, mainWindow->getActionIcon(MMainWindow::What_Logout)
	);

	_lockScreenItem = new QListViewItem(_actions,
		mainWindow->getActionName(MMainWindow::What_LockScreen));
	_lockScreenItem->setPixmap(
		0, mainWindow->getActionIcon(MMainWindow::What_LockScreen)
	);

	_rebootItem = new QListViewItem(_actions,
		mainWindow->getActionName(MMainWindow::What_Reboot));
	_rebootItem->setPixmap(
		0, mainWindow->getActionIcon(MMainWindow::What_Reboot)
	);

	_shutDownItem = new QListViewItem(_actions,
		mainWindow->getActionName(MMainWindow::What_ShutDown));
	_shutDownItem->setPixmap(
		0, mainWindow->getActionIcon(MMainWindow::What_ShutDown)
	);

	connect(
		_actions, SIGNAL(doubleClicked(QListViewItem *, const QPoint &, int)),
		this, SLOT(editActionSlot()));
	connect(
		_actions, SIGNAL(spacePressed(QListViewItem *)),
		this, SLOT(editActionSlot()));
	// edit action
	KPushButton *b_editAction = new KPushButton(SmallIcon("edit"),
		i18n("&Edit..."), gb_general, "KPushButton::b_editAction");
	connect(
		b_editAction, SIGNAL(clicked()),
		this, SLOT(editActionSlot()));
	// tip
	MTip *t_actions =
		new MTip(MTip::TipType_Normal, gb_general, "MTip::t_actions");
	t_actions->setTipText(
		"<ul>" \
		"<li>" + i18n("If you are using <b>KDE</b> and <b>KDM</b> (KDE Display Manager), then set all methods to <i>KDE</i>") + "</li>" \
		"<li>" + i18n("If you are using <b>KDE</b> and display manager different than <b>KDM</b>, then set <i>Turn Off Computer</i> and <i>Restart Computer</i> methods to <i>/sbin/...</i>") + "</li>" \
		"<li>" + i18n("If you are running <b>KShutDown</b> from the non-KDE session (e.g. <b>GNOME</b>), then change all methods...") + "</li>" \
		"<li>" + i18n("See FAQ for more details") + "</li>" \
		"</ul>"
	);
	l->addWidget(gb_general);

	QVGroupBox *gb_extras = new QVGroupBox(i18n("E&xtras Menu"), _actionsPage,
		"QVGroupBox::gb_extras");
	KPushButton *b_modifyExtras = new KPushButton(SmallIcon("edit"),
		i18n("&Modify..."), gb_extras, "KPushButton::b_modifyExtras");
	b_modifyExtras->setEnabled(
		MiscUtils::isActionEnabled(MMainWindow::What_Extras)
	);
	connect(
		b_modifyExtras, SIGNAL(clicked()),
		this, SLOT(modifyExtrasSlot()));
	l->addWidget(gb_extras);

	// update items after widgets init
	updateActions();
}

void MSettingsDialog::initAutoPage()
{
	_autoPage = addPage(i18n("Automation"), "reload");
	QVBoxLayout *l = new QVBoxLayout(_autoPage, 2);

	// after login
	QVGroupBox *gb_afterLogin = new QVGroupBox(i18n("A&fter Login"), _autoPage,
		"QVGroupBox::gb_afterLogin");
	c_autoLock = new QCheckBox(i18n("&Lock screen"), gb_afterLogin,
		"QCheckBox::c_autoLock");
	l->addWidget(gb_afterLogin);

	// before shut down
	QVGroupBox *gb_beforeShutDown =
		new QVGroupBox(i18n("&Before System Shut Down"), _autoPage,
		"QVGroupBox::gb_beforeShutDown");
	c_runCommandBeforeShutDown = new QCheckBox(i18n("&Run command"),
		gb_beforeShutDown, "QCheckBox::c_runCommandBeforeShutDown");
	in_commandBeforeShutDown = new KURLRequester(gb_beforeShutDown,
		"KURLRequester::in_commandBeforeShutDown");
	in_commandBeforeShutDown->setMode(KFile::File);
	in_commandPause =
		new KIntNumInput(gb_beforeShutDown, "KIntNumInput::in_commandPause");
	in_commandPause->setLabel(i18n("&Pause after run command:"));
	in_commandPause->setRange(0, 300, 1, TRUE);
	in_commandPause->setSuffix(" " + i18n("second(s)"));
	KPushButton *b_testCommandBeforeShutDown = new KPushButton(i18n("Test"),
		gb_beforeShutDown, "KPushButton::b_testCommandBeforeShutDown");
	connect(
		b_testCommandBeforeShutDown, SIGNAL(clicked()),
		this, SLOT(testCommandBeforeShutDownSlot()));
	l->addWidget(gb_beforeShutDown);

	// -
	l->addStretch();

	connect(
		c_runCommandBeforeShutDown, SIGNAL(toggled(bool)),
		in_commandBeforeShutDown, SLOT(setEnabled(bool)));
	connect(
		c_runCommandBeforeShutDown, SIGNAL(toggled(bool)),
		in_commandPause, SLOT(setEnabled(bool)));
	connect(
		c_runCommandBeforeShutDown, SIGNAL(toggled(bool)),
		b_testCommandBeforeShutDown, SLOT(setEnabled(bool)));
}

void MSettingsDialog::initConfirmPage()
{
	_confirmPage = addPage(i18n("Confirmations & Messages"),
		"messagebox_warning");
	QVBoxLayout *l = new QVBoxLayout(_confirmPage, 2);

	// confirms
	QVGroupBox *gb_confirms = new QVGroupBox(i18n("Co&nfirmations"),
		_confirmPage, "QVGroupBox::gb_confirms");
	// confirm action
	c_confirmAction = new QCheckBox(i18n("Confirm action (&recommended)"),
		gb_confirms, "QCheckBox::c_confirmAction");
	l->addWidget(gb_confirms);

	// popup messages
	QVGroupBox *gb_popup = new QVGroupBox(i18n("&Popup Messages (Passive)"),
		_confirmPage, "QVGroupBox::gb_popup");
	c_popupMessages =
		new QCheckBox(i18n("&Enabled"), gb_popup, "QCheckBox::c_popupMessages");
	// popup timeout
	in_popupTimeout =
		new KIntNumInput(gb_popup, "KIntNumInput::in_popupTimeout");
	in_popupTimeout->setLabel(i18n("Hide &message after:"));
	in_popupTimeout->setRange(2, 300, 1, TRUE);
	in_popupTimeout->setSuffix(" " + i18n("second(s)"));
	KPushButton *popupMessagesTest = new KPushButton(i18n("Test"), gb_popup,
		"KPushButton::popupMessagesTest");
	connect(
		popupMessagesTest, SIGNAL(clicked()),
		this, SLOT(popupMessagesTestSlot()));
	l->addWidget(gb_popup);

	// warning message:
	QVGroupBox *gb_warning =
		new QVGroupBox(i18n("&Warning Message (recommended)"), _confirmPage,
		"QVGroupBox::gb_warning");
	// enabled/info
	c_warningMessageEnabled =
		new QCheckBox("", gb_warning, "QCheckBox::c_warningMessageEnabled");
	// slider
	in_warningMessageDelay =
		new KIntNumInput(gb_warning, "KIntNumInput::in_warningMessageDelay");
	in_warningMessageDelay->setSuffix(" " + i18n("minute(s)"));
	in_warningMessageDelay->setRange(1, 60, 1, TRUE);
	connect(
		in_warningMessageDelay, SIGNAL(valueChanged(int)),
		this, SLOT(updateWarningMessageSlot()));
	KPushButton *warningMessageTest = new KPushButton(i18n("Test"), gb_warning,
		"KPushButton::warningMessageTest");
	connect(
		warningMessageTest, SIGNAL(clicked()),
		this, SLOT(warningMessageTestSlot()));
	l->addWidget(gb_warning);

	// notifications
	KPushButton *b_configureNotifications = new KPushButton(
		SmallIcon("knotify"),
		i18n("Con&figure Notifications..."),
		_confirmPage, "KPushButton::b_configureNotifications"
	);
	connect(
		b_configureNotifications, SIGNAL(clicked()),
		this, SLOT(configureNotificationsSlot()));
	l->addWidget(b_configureNotifications);

	// re-enable messages
	KPushButton *b_enableAllMessages = new KPushButton(
		i18n("Enable All Me&ssages"),
		_confirmPage, "KPushButton::b_enableAllMessages"
	);
	QWhatsThis::add(
		b_enableAllMessages,
		"<qt>" +
		i18n(
			"Enable all messages which have been turned off with the " \
			"<b>Do not show this message again</b> feature."
		) +
		"</qt>"
	);
	connect(
		b_enableAllMessages, SIGNAL(clicked()),
		this, SLOT(enableAllMessagesSlot()));
	l->addWidget(b_enableAllMessages);

	// -
	l->addStretch();

	connect(
		c_popupMessages, SIGNAL(toggled(bool)),
		in_popupTimeout, SLOT(setEnabled(bool)));
	connect(
		c_popupMessages, SIGNAL(toggled(bool)),
		popupMessagesTest, SLOT(setEnabled(bool)));
	connect(
		c_warningMessageEnabled, SIGNAL(toggled(bool)),
		in_warningMessageDelay, SLOT(setEnabled(bool)));
	connect(
		c_warningMessageEnabled, SIGNAL(toggled(bool)),
		warningMessageTest, SLOT(setEnabled(bool)));
	updateWarningMessageSlot();
}

void MSettingsDialog::initGeneralPage()
{
	_generalPage = addPage(i18n("General"), "misc");
	QVBoxLayout *l = new QVBoxLayout(_generalPage, 2);

	// remember time settings
	c_timeRemember = new QCheckBox(i18n("Remember time &settings"),
		_generalPage, "QCheckBox::c_timeRemember");
	l->addWidget(c_timeRemember);

	// kde settings
	QVGroupBox *gb_kde = new QVGroupBox(i18n("&KDE Settings"), _generalPage,
		"QVGroupBox::gb_kde");
	// lock screen settings
	b_lockScreenSettings = new KPushButton(SmallIcon("kscreensaver"),
		i18n("Screen Sa&ver..."), gb_kde, "KPushButton::b_lockScreenSettings");
	connect(
		b_lockScreenSettings, SIGNAL(clicked()),
		this, SLOT(lockSettingsSlot()));
	// logout settings
	b_logoutSettings = new KPushButton(i18n("Session &Manager..."), gb_kde,
		"KPushButton::b_logoutSettings");
	connect(
		b_logoutSettings, SIGNAL(clicked()),
		this, SLOT(logoutSettingsSlot()));
	l->addWidget(gb_kde);

	// links
	QVGroupBox *gb_links = new QVGroupBox(i18n("&Links"), _generalPage);
	new Links(gb_links, "Links");
	l->addWidget(gb_links);

	QString karambaThemes = locate("appdata", "karamba/kshutdown-32x32.png");
	if (!karambaThemes.isNull())
	{
		int i = karambaThemes.findRev("/");
		if (i != -1 && i > 0)
		{
			MTip *t_karamba = new MTip(MTip::TipType_Info, _generalPage);
			karambaThemes = karambaThemes.left(i);
			t_karamba->setTipText(
				"<a href=\"" + karambaThemes + "\">" +
				i18n("<b>Karamba</b> Themes") + "</a><br>" \
				"(" + karambaThemes + ")"
			);
			l->addWidget(t_karamba);
		}
	}

	// -
	l->addStretch();
}

void MSettingsDialog::initNotifyPage()
{
/* TODO: 0.5:
	_notifyPage = addPage(
		"Notifications",
		"knotify"
	);
	QVBoxLayout *l = new QVBoxLayout(_notifyPage, 2);
	l->addWidget(_notifyWidget);

// temporarily copied from k3b
K3bNotifyOptionTab::K3bNotifyOptionTab( QWidget* parent, const char* name )
  : QWidget( parent, name )
{
  m_notifyWidget = new KNotify::KNotifyWidget( this );

  QHBoxLayout* box = new QHBoxLayout( this );
  box->addWidget( m_notifyWidget );
}

void K3bNotifyOptionTab::readSettings()
{
  m_notifyWidget->clear();
  KNotify::Application* app = m_notifyWidget->addApplicationEvents( "k3b/eventsrc" );
  if( app )
    m_notifyWidget->addVisibleApp(app);
  else
    kdDebug() << "(K3bNotifyOptionTab) could not find K3b eventsrc." << endl;
}

bool K3bNotifyOptionTab::saveSettings()
{
  m_notifyWidget->save();

  return true;
}
*/
}

void MSettingsDialog::initSchedulerPage()
{
	_schedulerPage = addPage(i18n("Scheduler"), "misc");
	QVBoxLayout *l = new QVBoxLayout(_schedulerPage, 2);

	// confirm action
	c_schedulerEnabled = new QCheckBox(i18n("&Enabled"), _schedulerPage,
		"QCheckBox::c_schedulerEnabled");
	l->addWidget(c_schedulerEnabled);

	// -
	l->addStretch();
}

void MSettingsDialog::initStatsPage()
{
	_statsPage = addPage(i18n("Statistics"), "misc");
	QVBoxLayout *l = new QVBoxLayout(_statsPage, 2);

	// long format
	c_statsLongFormat =
		new QCheckBox(i18n("&Show login time, JCPU and PCPU times"), _statsPage,
		"QCheckBox::c_statsLongFormat");
	l->addWidget(c_statsLongFormat);

	QVGroupBox *gb_colors =
		new QVGroupBox(i18n("Co&lors"), _statsPage, "QVGroupBox::gb_colors");

	QLabel *label;

	// text color
	QHBox *textBox = new QHBox(gb_colors);
	textBox->setSpacing(2);
	label = new QLabel(i18n("Text color:"), textBox);
	label->setAlignment(AlignRight | AlignVCenter);
	b_statsText = new KColorButton(textBox);

	// background color
	QHBox *backgroundBox = new QHBox(gb_colors);
	backgroundBox->setSpacing(2);
	label = new QLabel(i18n("Background color:"), backgroundBox);
	label->setAlignment(AlignRight | AlignVCenter);
	b_statsBackground = new KColorButton(backgroundBox);

	// header color
	QHBox *headerBox = new QHBox(gb_colors);
	headerBox->setSpacing(2);
	label = new QLabel(i18n("Header color:"), headerBox);
	label->setAlignment(AlignRight | AlignVCenter);
	b_statsHeader = new KColorButton(headerBox);

	// warning color
	QHBox *warningBox = new QHBox(gb_colors);
	warningBox->setSpacing(2);
	label = new QLabel(i18n("Warning color:"), warningBox);
	label->setAlignment(AlignRight | AlignVCenter);
	b_statsWarning = new KColorButton(warningBox);

	// preview
	new QLabel(i18n("Preview:"), gb_colors);
	_statsPreview = new QTextEdit(gb_colors);
	_statsPreview->setReadOnly(TRUE);
	_statsPreview->setTextFormat(RichText); // HTML tags
	MiscUtils::setHint(_statsPreview, i18n("Preview"));

	l->addWidget(gb_colors);

	// -
	l->addStretch();

	connect(
		b_statsBackground, SIGNAL(changed(const QColor &)),
		this, SLOT(statsColorChangedSlot(const QColor &)));
	connect(
		b_statsHeader, SIGNAL(changed(const QColor &)),
		this, SLOT(statsColorChangedSlot(const QColor &)));
	connect(
		b_statsText, SIGNAL(changed(const QColor &)),
		this, SLOT(statsColorChangedSlot(const QColor &)));
	connect(
		b_statsWarning, SIGNAL(changed(const QColor &)),
		this, SLOT(statsColorChangedSlot(const QColor &)));

	updateStatsPreview();
}

void MSettingsDialog::setActionMethod(const QString &configEntry,
	const MMainWindow::Method method, const QString &command) const
{
	KConfig *config = kapp->config();
	config->setGroup(configEntry);
	if (method == MMainWindow::Method_UserCommand)
		config->writeEntry("Command", command);
	config->writeEntry("Method", method);
}

void MSettingsDialog::setWarningMessage(const bool on, const int delay)
{
	in_warningMessageDelay->setEnabled(on);
	in_warningMessageDelay->setValue(delay);
	c_warningMessageEnabled->setChecked(on);
}

void MSettingsDialog::updateActions()
{
	updateItem(_shutDownItem);
	updateItem(_rebootItem);
	updateItem(_lockScreenItem);
	updateItem(_logoutItem);
	_actions->setCurrentItem(_shutDownItem);
}

void MSettingsDialog::updateItem(QListViewItem *item)
{
	if (!item)
		return;

	MMainWindow::Method m = MMainWindow::Method_KDE;
	QString c;

	if (item == _shutDownItem)
	{
		mainWindow->getMethod("Shut Down", m, c, DEFAULT_SHUT_DOWN_COMMAND);
	}
	else if (item == _rebootItem)
	{
		mainWindow->getMethod("Reboot", m, c, DEFAULT_REBOOT_COMMAND);
	}
	else if (item == _lockScreenItem)
	{
		mainWindow->getMethod("Lock Screen", m, c, DEFAULT_LOCK_SCREEN_COMMAND);
	}
	else if (item == _logoutItem)
	{
		mainWindow->getMethod("Logout", m, c, DEFAULT_LOGOUT_COMMAND);
	}

	// method name
	if (m == MMainWindow::Method_KDE)
		item->setText(1, "KDE");
	else
		item->setText(1, c);
}

void MSettingsDialog::updateStatsPreview()
{
	QString s = "<qt><pre style=\"font-family: Luxi Mono, Courier\"><b>";
	s.append("<font color=\"" + b_statsText->color().name() + "\">" \
		"02:02:16 up 18 min,  2 users,  load average: 1,01, 0,91, 0,71" \
		"</font><br>");
	s.append("<font color=\"" + b_statsHeader->color().name() + "\">" \
		"USER     TTY      FROM         IDLE  WHAT" \
		"</font><br>");
	s.append("<font color=\"" + b_statsText->color().name() + "\">" \
		"kdt      pts/1    -           16:06  kdeinit: kwrited" \
		"</font><br>");
	s.append("<font color=\"" + b_statsText->color().name() + "\">" \
		"<font color=\"" + b_statsWarning->color().name() + "\">root</font>" \
		"     tty1     -            3:39  -bash" \
		"</font><br>");
	s.append("</b></pre></qt>");

	_statsPreview->setText(s);
	_statsPreview->setPaletteBackgroundColor(b_statsBackground->color());
	_statsPreview->setPaletteForegroundColor(b_statsText->color());
}

/* private slots */

void MSettingsDialog::configureNotificationsSlot()
{
	KNotifyDialog::configure(mainWindow);
}

void MSettingsDialog::editActionSlot()
{
	QListViewItem *item = _actions->currentItem();

	if (!item)
		return;

	MMainWindow::Method m = MMainWindow::Method_KDE;
	QString c;

	MActionEditDialog *dialog =
		new MActionEditDialog(this, "MActionEditDialog::dialog");

	Q_CHECK_PTR(dialog);

	dialog->methods->insertItem("KDE");

	if (item == _shutDownItem)
	{
		mainWindow->getMethod("Shut Down", m, c, DEFAULT_SHUT_DOWN_COMMAND);
		dialog->methods->insertItem(DEFAULT_SHUT_DOWN_COMMAND);
		dialog->setAction(MMainWindow::What_ShutDown);
	}
	else if (item == _rebootItem)
	{
		mainWindow->getMethod("Reboot", m, c, DEFAULT_REBOOT_COMMAND);
		dialog->methods->insertItem(DEFAULT_REBOOT_COMMAND);
		dialog->setAction(MMainWindow::What_Reboot);
	}
	else if (item == _lockScreenItem)
	{
		mainWindow->getMethod("Lock Screen", m, c, DEFAULT_LOCK_SCREEN_COMMAND);
		dialog->methods->insertItem(DEFAULT_LOCK_SCREEN_COMMAND);
		dialog->setAction(MMainWindow::What_LockScreen);
	}
	else if (item == _logoutItem)
	{
		mainWindow->getMethod("Logout", m, c, DEFAULT_LOGOUT_COMMAND);
		dialog->methods->insertItem(DEFAULT_LOGOUT_COMMAND);
		dialog->setAction(MMainWindow::What_Logout);
	}
	dialog->methods->insertItem(i18n("User Command"));

	dialog->setUserCommand(c);
	dialog->setActionMethod(m);

	if (dialog->exec() == QDialog::Accepted)
	{
		m = (MMainWindow::Method)dialog->methods->currentItem();
		if (m == MMainWindow::Method_UserCommand)
			c = dialog->command->text();

		if (item == _shutDownItem)
		{
			setActionMethod("Shut Down", m, c);
		}
		else if (item == _rebootItem)
		{
			setActionMethod("Reboot", m, c);
		}
		else if (item == _lockScreenItem)
		{
			setActionMethod("Lock Screen", m, c);
		}
		else if (item == _logoutItem)
		{
			setActionMethod("Logout", m, c);
		}

		updateItem(item);
	}

	delete dialog;
}

void MSettingsDialog::enableAllMessagesSlot()
{
	KMessageBox::enableAllMessages();
}

void MSettingsDialog::enableLockSettingsButtonSlot()
{
	b_lockScreenSettings->setEnabled(TRUE);
}

void MSettingsDialog::enableLogoutSettingsButtonSlot()
{
	b_logoutSettings->setEnabled(TRUE);
}

void MSettingsDialog::lockSettingsSlot()
{
	b_lockScreenSettings->setEnabled(FALSE);
	QTimer::singleShot(5000, this, SLOT(enableLockSettingsButtonSlot()));
	KRun::run("kcmshell screensaver", KURL::List());
}

void MSettingsDialog::logoutSettingsSlot()
{
	b_logoutSettings->setEnabled(FALSE);
	QTimer::singleShot(5000, this, SLOT(enableLogoutSettingsButtonSlot()));
	KRun::run("kcmshell kcmsmserver", KURL::List());
}

void MSettingsDialog::modifyExtrasSlot()
{
	mainWindow->extras()->modify();
}

void MSettingsDialog::okClickedSlot()
{
	// general
	kshutdownrc->timeRemember = c_timeRemember->isChecked();

	// automation
	if (c_autoLock->isChecked())
	{
		Links::createLink(
			KGlobalSettings::autostartPath() + "/" + AUTO_LOCK_FILE_NAME,
			"kshutdown --init --lock",
			"kshutdown"
		);
	}
	else
	{
		Links::removeLink(
			KGlobalSettings::autostartPath() + "/" + AUTO_LOCK_FILE_NAME
		);
	}

	kshutdownrc->runCommandBeforeShutDown =
		c_runCommandBeforeShutDown->isChecked();
	kshutdownrc->commandBeforeShutDown = in_commandBeforeShutDown->url();
	kshutdownrc->commandPause = in_commandPause->value();

	// confirmations
	kshutdownrc->confirmAction = c_confirmAction->isChecked();
	KConfig *conf = kapp->config();
	conf->setGroup("Notification Messages");
	if (kshutdownrc->confirmAction)
		conf->deleteEntry("ShutDown");
	else
		conf->writeEntry("ShutDown", "yes");
	kshutdownrc->popupMessages = c_popupMessages->isChecked();
	kshutdownrc->popupTimeout = in_popupTimeout->value();
	kshutdownrc->warningMessageEnabled = c_warningMessageEnabled->isChecked();
	kshutdownrc->warningMessageDelay = in_warningMessageDelay->value();

	// scheduler
	kshutdownrc->schedEnabled = c_schedulerEnabled->isChecked();

	// statistics
	kshutdownrc->statsLongFormat = c_statsLongFormat->isChecked();
	kshutdownrc->statsBackground = b_statsBackground->color();
	kshutdownrc->statsHeader = b_statsHeader->color();
	kshutdownrc->statsText = b_statsText->color();
	kshutdownrc->statsWarning = b_statsWarning->color();
}

void MSettingsDialog::popupMessagesTestSlot()
{
	mainWindow->tray()->popupMessage(
		MSystemTray::Reason_Info,
		i18n("Test"),
		i18n("Test"),
		in_popupTimeout->value(), // timeout
		TRUE // test
	);
}

void MSettingsDialog::statsColorChangedSlot(const QColor &/*newColor*/)
{
	updateStatsPreview();
}

void MSettingsDialog::testCommandBeforeShutDownSlot()
{
	MiscUtils::runCommandWithPause(
		in_commandBeforeShutDown->url(),
		in_commandPause->value()
	);
}

void MSettingsDialog::updateWarningMessageSlot()
{
	QString s =
		QString(i18n("Show warning message %1 minute(s) &before action"))
			.arg(in_warningMessageDelay->value());
	c_warningMessageEnabled->setText(s);
}

void MSettingsDialog::warningMessageTestSlot()
{
	mainWindow->setCancelled(FALSE);
	MMessageDialog *messageDialog = new MMessageDialog(
		TRUE, in_warningMessageDelay->value(),
		MMainWindow::What_Nothing, i18n("Test"),
		this, "MMessageDialog::messageDialog"
	);

	Q_CHECK_PTR(messageDialog);

	messageDialog->exec();
	delete messageDialog;
}

/* protected slots */

void MSettingsDialog::slotDefault()
{
	int index = activePageIndex();

	if (index == -1)
		return;

	// general
	if (index == pageIndex(_generalPage))
	{
		c_timeRemember->setChecked(DEFAULT_TIME_REMEMBER);
	}
	// automation
	else if (index == pageIndex(_autoPage))
	{
		c_autoLock->setChecked(DEFAULT_AUTO_LOCK);
		c_runCommandBeforeShutDown->setChecked(
			DEFAULT_RUN_COMMAND_BEFORE_SHUT_DOWN
		);
		in_commandPause->setValue(DEFAULT_COMMAND_PAUSE);
	}
	// confirmations
	else if (index == pageIndex(_confirmPage))
	{
		c_confirmAction->setChecked(DEFAULT_CONFIRM_ACTION);
		c_popupMessages->setChecked(DEFAULT_POPUP_MESSAGES);
		in_popupTimeout->setValue(DEFAULT_POPUP_TIMEOUT);
		setWarningMessage(
			DEFAULT_WARNING_MESSAGE_ENABLED,
			DEFAULT_WARNING_MESSAGE_DELAY
		);
	}
	// actions
	else if (index == pageIndex(_actionsPage))
	{
		if (disabledByAdmin("actions"))
			return;

		if (KMessageBox::warningYesNo(
			this,
			i18n("Restore default settings for this page?"),
			caption()) != KMessageBox::Yes
		) /* -> */ return;

		KConfig *config = kapp->config();
		config->deleteGroup("Shut Down");
		config->deleteGroup("Reboot");
		config->deleteGroup("Lock Screen");
		config->deleteGroup("Logout");
		updateActions();
	}
	// scheduler
	else if (index == pageIndex(_schedulerPage))
	{
		c_schedulerEnabled->setChecked(DEFAULT_SCHED_ENABLED);
	}
	// stats
	else if (index == pageIndex(_statsPage))
	{
		c_statsLongFormat->setChecked(DEFAULT_STATS_LONG_FORMAT);
		b_statsBackground->setColor(QColor(DEFAULT_STATS_BACKGROUND));
		b_statsHeader->setColor(QColor(DEFAULT_STATS_HEADER));
		b_statsText->setColor(QColor(DEFAULT_STATS_TEXT));
		b_statsWarning->setColor(QColor(DEFAULT_STATS_WARNING));
	}
}
